const Fastify = require("fastify");
const fs = require("fs");

module.exports = class PCCApp {
  constructor(appName) {
    // Path to the config file
    const conf_file_Path = `/etc/picockpit-client/svc.d/${appName}.json`;
    var port;
    var jsonData;
    this.fastify = Fastify();

    try {
      const data = fs.readFileSync(conf_file_Path, "utf8");
      var jsonData = JSON.parse(data);
    } catch {
      console.error(
        `Error reading configuration file. \n it should be located at /etc/picockpit-client/svc.d/${appName}.json\n` +
          "There is probably an issue with missing or corrputed configuration files.\n" +
          "Try reinstalling Picockpit from a new .deb file, either via the Website or via the Repository.\nContact our team at www.picockpit.com if the error persists.",
        err,
      );
      // We simply exit here since the systemd service takes care of restarting & testing again.
      process.exit(1);
    }
    try {
      port = jsonData["com_port"];
    } catch {
      console.error(
        `Error parsing configuration file. \n it should be located at /etc/picockpit-client/svc.d/${appName}.json\n` +
          "There is probably an issue with missing or corrputed configuration files.\n" +
          "Try reinstalling Picockpit from a new .deb file, either via the Website or via the Repository.\nContact our team at www.picockpit.com if the error persists.",
        err,
      );
      // We simply exit here since the systemd service takes care of restarting & testing again.
      process.exit(1);
    }

    this.fastify.listen({ port: port }, function (err, address) {
      console.log(`Starting up the ${appName} on port: ${port}`);
      if (err) {
        console.error(
          "There has been an error starting up or running the server for picockpit-internal communication.\n" +
            `This is probably an issue with ports, the server starts up on the port defined inside /etc/picockpit-client/svc.d/${appName}.json\n` +
            "You can double check if this port is already occupied on your system.\n" +
            "Should this not be the case please write us on picockpit.com for further help.",
        );
        // Note that this error doesn't include a message for the server - if the config file is missing we obviously can't reach the server.
        this.fastify.log.error(err);
        process.exit(1);
      }
    });
  }
};
