import PCCApp from "../picockpit-library/picockpit-wapi-base.js";

// check out @homebridge/node-pty-prebuilt-multiarch  they have triple the number of dl
// or the microsoft one

import * as pty from "node-pty";
import nodeDatachannelPolyfill from "node-datachannel/polyfill";
import Peer from "simple-peer";
// add code for handling multiple connects, disconnects (free up the pty and simplepeer obj so gc can happen)
// ex if peer dies, code exists with error
// NOTE: Node has no native wrtc libs.  Existing ones mostly lack vid/audio support since
// node also lacks any browser-like mediaquery API.
// NOTE: node-pty IS NOT worker/thread safe
// ISSUE: The normal wrtc module (nodejs stubs for googles wrtc binary):
// does not compile on node v16
// v16 is needed for pty
// does not work on arm
// has security issues and hasn't been updated in years
// Hence the polyfill
// simple-peer is used to cut the signaling calls in half.
// MediaSoup & Socket.io MIGHT be able to do some video/audio work on an arm node server if that is needed in the future.
// TODO
// exit -> pty, peer object destruction

let spConfig = null;
let termData = {};
let curNum = 0;

class terminal extends PCCApp {
  constructor() {
    super("terminal");

    this.fastify.post("/whep", function (req, res) {
      try {
        console.log("post", JSON.stringify(req.body));
        // start a new terminal
        curNum = curNum + 1;
        const term = new Term(spConfig, curNum);
        termData[curNum] = term;
        // init answer cb
        term.peerConnection.on("signal", (data) => {
          res.send(data);
        });
        // start answer process
        term.peerConnection.signal(req.body);
      } catch (err) {
        console.error(
          "There was an error while creating a new terminal. The whole process will be restarted.",
        );
        console.error("This way we can be sure that no bad state persists.");
        console.error(err);
        res.send({
          statusCode: 501,
          message: "Error while initializing the ptyProcess.",
        });
        process.exit(1);
      }
    });

    this.fastify.post("/cred", function (req, res) {
      try {
        spConfig = req.body;
      } catch (err) {
        error.log("Error while setting the configuration.");
        res.send({
          statusCode: 501,
          message: "Error while initializing the configuration.",
        });
      }
    });
  }
}

class Term {
  constructor(wrtcCfg, curNum) {
    this.wrtc_connected = false;
    this.id = curNum;
    // shell init
    const shell = "bash";
    try {
      this.ptyProcess = pty.spawn(shell, [], {
        name: "alacritty",
        cols: 80,
        rows: 30,
        cwd: "/",
        env: process.env,
      });
      console.log("ptypid:", this.ptyProcess.pid); // readlink /proc/PID/cwd
    } catch (err) {
      throw new Error("Error while creating the PTY process: ", err);
    }

    this.ptyProcess.onData((data) => {
      if (this.wrtc_connected) this.peerConnection.send(data);
    });
    this.peerConnection = new Peer(
      { wrtc: nodeDatachannelPolyfill, trickle: false },
      wrtcCfg,
    );

    this.peerConnection.on("data", (data) => {
      try {
        console.log("got a message from peer1: " + data.toString());
        if (data.length > 1) {
          // event
          const resizeRegex = /resizeEvent\((\d+),(\d+)\)/;
          if (resizeRegex.test(data)) {
            // resize
            const s = data.toString().match(resizeRegex);
            console.log("resize", s);
            // check arg len too!
            this.ptyProcess.resize(parseInt(s[1]), parseInt(s[2]));
            return;
          }
          console.log("writing to pty");
          this.ptyProcess.write(data);
        } else this.ptyProcess.write(data);
      } catch (err) {
        throw new Error("Error while processing: ", err);
      }
    });
    this.peerConnection.on("connect", () => {
      this.wrtc_connected = true;
      console.log("CONNECTED TO PEER");
    });
    this.peerConnection.on("close", () => {
      console.log(Object.keys(termData).length);
      delete termData[this.id];
      console.log("CLOSED BY PEER");
    });
    this.peerConnection.on("error", () => {
      delete termData[this.id];
      console.log("ERROR BY PEER");
    });
    // Fill these out
    // HANNO. THIS IS BAD. MAKE PROPER GETTERS/SETTERS
    return this;
  }
}

terminal = new terminal();
